<?php namespace EwCacheInvalidator;

use WP_Query;
use OcUtilities;
use EwCacheInvalidator\DataStorage;
use EwCacheInvalidator\PageQueries;
use EwCacheInvalidator\Lcc;
use EwCacheInvalidator\LedgeInvalidator;

/**
 * Class EventHandler
 * @package EwCacheInvalidator
 */
class EventHandler {

    /**
     * EventHandler constructor.
     */
    public function __construct() {

        /**
         * Triggers on post save, check if we should update page data.
         */
        add_action('save_post', function($post_id) {

            $postType = get_post_type($post_id);
            if($postType === 'page' && get_post_status($post_id) !== 'trash') {
                $this->pageHasBeenUpdated($post_id);
            }

            return;
        });

        /**
         * Triggers on post sent to trash, check if we should update page data.
         */
        add_action('trashed_post', function($post_id) {
            $this->postHasBeenDeleted($post_id);
            return;
        });

        /**
         * Triggers on post deleted, check if we should update page data.
         */
        add_action('deleted_post', function($post_id) {
            $this->postHasBeenDeleted($post_id);
            return;
        });

        /**
         * Triggers on board updated, update page data.
         */
        add_action('board_changed_hook', function($page_id) {
            $this->pageHasBeenUpdated($page_id);
        });

        /**
         * Triggers on widget update, update data for pages where widget is active.
         */
        add_filter('widget_update_callback', function($instance, $new, $old, $obj) {

            if(method_exists($obj, 'get_oc_query')) {

                $oldQuery = PageQueries::getQueryFromWidgetInstance($obj->id, $old);
                $newQuery = PageQueries::getQueryFromWidgetInstance($obj->id, $new);

                if($newQuery !== $oldQuery) {

                    $dataStorage = new DataStorage();
                    $lcc = new Lcc();
                    $boards = $this->getBoardsForWidget($obj->id);

                    foreach($boards as $board) {
                        $boardPageIds = $this->getPagesConnectedToBoard($board['id']);
                        if(count($boardPageIds) > 0) {
                            foreach($boardPageIds as $boardPageId) {

                                $pageItems = $dataStorage->getItemsByPageId($boardPageId);
                                foreach($pageItems as $item) {
                                    if($oldQuery === $item->oc_query) {
                                        $lcc->deregisterListener($item->listener_id);
                                        $dataStorage->deleteItem($item->id);
                                    }
                                }

                                $listenerId = $lcc->registerListener($newQuery);
                                $dataStorage->saveItem(new QueryItem($newQuery, $boardPageId, 'widget', $listenerId));
                            }
                        }
                    }
                }
            }

            return $instance;
        }, 10, 4);

        /**
         * Triggers on oc update for articles.
         */
        add_action('oc_push_article_update', function($data) {
            if(isset($data['uuid'])) {
                $postId = OcUtilities::get_article_post_id_by_uuid($data['uuid']);
                if(isset($postId)) {
                    LedgeInvalidator::invalidateUrl(get_permalink($postId));
                }
            }
        });

        /**
         * Triggers on update for lists.
         */
        add_action('oc_push_list_update', function($data) {
            if(isset($data['uuid'])) {
                $page_ids = $this->getPageIdsForOcList($data['uuid']);
                foreach($page_ids as $page_id) {
                    $this->pageHasBeenUpdated($page_id);
                }
            }
        });
    }

    /**
     * Post has been deleted, update query data for pages.
     *
     * @param $post_id
     */
    private function postHasBeenDeleted($post_id) {

        $postType = get_post_type($post_id);
        if($postType === 'page') {
            $this->pageHasBeenDeleted($post_id);
        }
        else if($postType === 'board') {

            $boardPageIds = $this->getPagesConnectedToBoard($post_id);
            if($boardPageIds !== false) {
                foreach($boardPageIds as $boardPageId) {
                    $this->pageHasBeenDeleted($boardPageId);
                }
            }
        }
    }

    /**
     * Page has been updated, update query data for page.
     *
     * @param $page_id
     */
    private function pageHasBeenUpdated($page_id) {

        $lcc = new Lcc();
        $dataStorage = new DataStorage();
        $existingPageItems = $dataStorage->getItemsByPageId($page_id);

        $pageQueries = new PageQueries();
        $updatedPageItems = $pageQueries->getPageQueries($page_id);
        $newPageItems = [];

        foreach($updatedPageItems as $query) {

            if(is_array($query) && count($query) >= 3) {
                $newPageItems[] = new QueryItem($query[0], $query[1], $query[2]);
            }
        }

        foreach($newPageItems as $item) {
            $removeItem = null;
            foreach($existingPageItems as $existingItem) {

                if($item->oc_query === $existingItem->oc_query && $item->type === $existingItem->type) {
                    $item->id = $existingItem->id;
                    $item->listener_id = $existingItem->listener_id;
                }
            }
        }

        foreach($newPageItems as $newItem) {

            if(!isset($newItem->id) || $newItem->id === 0) {

                $listener_id = $lcc->registerListener($newItem->oc_query);
                if($listener_id !== false) {
                    $newItem->listener_id = $listener_id;
                    $dataStorage->saveItem($newItem);
                }
            }
        }

        foreach($existingPageItems as $existingItem) {

            $itemExists = false;
            foreach($newPageItems as $newItem) {

                if($existingItem->oc_query === $newItem->oc_query && $existingItem->type === $newItem->type) {
                    $itemExists = true;
                }
            }

            if(!$itemExists) {
                $lcc->deregisterListener($existingItem->listener_id);
                $dataStorage->deleteItem($existingItem->id);
            }
        }
    }

    /**
     * Page has been deleted, delete query data for page.
     *
     * @param $page_id
     */
    private function pageHasBeenDeleted($page_id) {

        $dataStorage = new DataStorage();
        $pageItems = $dataStorage->getItemsByPageId($page_id);

        if($pageItems !== false) {

            $lcc = new Lcc();
            foreach ($pageItems as $item) {
                $lcc->deregisterListener($item->listener_id);
            }

            $dataStorage->deleteItemsByPageId($page_id);
        }
    }

    /**
     * Get boards where widget id is active.
     *
     * @param $widget_id
     * @return array
     */
    private function getBoardsForWidget($widget_id) {

        $boardsArr = [];
        $board_args = array(
            'posts_per_page' => -1,
            'post_type'  => 'everyboard',
            'post_status' => 'any'
        );

        $boards = get_posts($board_args);
        $boards_data = [];

        foreach($boards as $board) {
            $json = get_post_meta($board->ID, 'everyboard_json', true);
            $boards_data[$board->ID]['title'] = $board->post_title;
            $boards_data[$board->ID]['json'] = $json;
        }

        array_walk($boards_data, function($board, $key) use (&$boardsArr, $widget_id) {
            if(false !== strpos($board['json'], $widget_id)) {
                $boardsArr[] = [
                    'id' => $key,
                    'title' => $board['title']
                ];
            }
        });

        return $boardsArr;
    }

    /**
     * Get page ids where the list uuid is used.
     *
     * @param $list_uuid
     * @return array
     */
    private function getPageIdsForOcList($list_uuid) {

        $board_args = array(
            'posts_per_page' => -1,
            'post_type'  => 'everyboard',
            'post_status' => 'any'
        );

        $board_posts = get_posts( $board_args );
        $boards_data = [];

        foreach($board_posts as $board) {
            $json = get_post_meta($board->ID, 'everyboard_json', true);
            $boards_data[$board->ID]['json'] = $json;
        }

        $board_ids = [];
        foreach($boards_data as $id => $board) {
            if( false !== strpos($board['json'], $list_uuid) ) {
                $board_ids[] = $id;
            }
        }

        if(count($board_ids) === 0) {
            return [];
        }

        $page_ids = [];
        foreach($board_ids as $board_id) {
            $page_ids = array_merge($page_ids, $this->getPagesConnectedToBoard($board_id));
        }

        return $page_ids;
    }

    /**
     * Get pages that are connected to given board id.
     *
     * @param $board_id
     * @return array
     */
    private function getPagesConnectedToBoard($board_id) {

        $args = array(
            'post_type'  => 'page',
            'meta_query' => array(
                array(
                    'key'     => 'everyboard_id',
                    'value'   => $board_id,
                    'compare' => '='
                )
        ));

        $posts = get_posts( $args );
        $returnArr = [];
        foreach( $posts as $page ) {
            $returnArr[] = $page->ID;
        }

        return $returnArr;
    }
}